'''
Created on 2016/1/5

:author: hubo
'''

from vlcp.protocol import Protocol
from vlcp.config import defaultconfig
from vlcp.event import Event, withIndices, ConnectionWriteEvent
import logging
import itertools
from vlcp.event.lock import Lock
from contextlib import closing
try:
    import hiredis
    hiredis_available = True
except:
    hiredis_available = False

@withIndices('state', 'connection', 'connmark', 'createby')
class RedisConnectionStateEvent(Event):
    CONNECTION_UP = 'up'
    CONNECTION_DOWN = 'down'
    CONNECTION_NOTCONNECTED = 'notconnected'

@withIndices('connection', 'connmark', 'id', 'iserror', 'createby')
class RedisResponseEvent(Event):
    pass

@withIndices('type', 'subscribe', 'connection', 'connmark', 'createby')
class RedisSubscribeEvent(Event):
    SUBSCRIBE = 'subscribe'
    UNSUBSCRIBE = 'unsubscribe'
    PSUBSCRIBE = 'psubscribe'
    PUNSUBSCRIBE = 'punsubscribe'

@withIndices('type', 'subscribe', 'channel', 'connection', 'connmark', 'createby')
class RedisSubscribeMessageEvent(Event):
    MESSAGE = 'message'
    PMESSAGE = 'pmessage'

class RedisProtocolException(Exception):
    pass

class RedisReplyException(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)
        if args:
            self.subtype, _, self.describe = args[0].partition(' ')
        else:
            self.subtype = ''
            self.describe = ''

def _copy(buffer):
    try:
        if isinstance(buffer, memoryview):
            return buffer.tobytes()
        else:
            return buffer[:]
    except:
        return buffer[:]

def _str(b, encoding = 'ascii'):
    if isinstance(b, str):
        return b
    elif isinstance(b, bytes):
        return b.decode(encoding)
    else:
        return str(b)

class RedisParser(object):
    "Python implemented hiredis.Reader()"
    def __init__(self):
        self._buffer = b''
        self._parser = self._parser_gen()
    def feed(self, data):
        self._buffer += data
    def gets(self):
        try:
            return next(self._parser)
        except StopIteration:
            raise RedisProtocolException(*self._exceptionargs)
        except RedisProtocolException as exc:
            self._exceptionargs = exc.args
            raise
    def _parser_gen(self, startpos = 0):
        try:
            _blk_buffer = []
            while True:
                lastfind = startpos
                while True:
                    le = self._buffer.find(b'\r', lastfind, -1)
                    if le < 0:
                        lastfind = len(self._buffer) - 1
                        if lastfind < startpos:
                            lastfind = startpos
                        if startpos > 0:
                            self._buffer = self._buffer[startpos:]
                            lastfind -= startpos
                            startpos = 0
                        yield False
                    else:
                        break
                c = self._buffer[startpos : startpos + 1]
                if c == b'+':
                    r = startpos + 1
                    startpos = le + 2
                    yield self._buffer[r : le]
                elif c == b'-':
                    r = startpos + 1
                    startpos = le + 2
                    yield RedisReplyException(_str(self._buffer[r : le]))
                elif c == b':':
                    r = startpos + 1
                    startpos = le + 2
                    try:
                        yield int(self._buffer[r : le])
                    except ValueError as exc:
                        raise RedisProtocolException(str(exc))
                elif c == b'$':
                    try:
                        blksize = int(self._buffer[startpos + 1 : le])
                    except ValueError as exc:
                        raise RedisProtocolException(str(exc))
                    startpos = le + 2
                    if blksize < 0:
                        yield None
                    else:
                        while len(self._buffer) - startpos < blksize:
                            _blk_buffer.append(self._buffer[startpos:])
                            blksize -= len(self._buffer) - startpos
                            self._buffer = b''
                            startpos = 0
                            yield False
                        _blk_buffer.append(self._buffer[startpos:blksize+startpos])
                        startpos += blksize
                        while len(self._buffer) - startpos < 2:
                            yield False
                        startpos += 2
                        r = b''.join(_blk_buffer)
                        del _blk_buffer[:]
                        yield r
                elif c == b'*':
                    try:
                        arraysize = int(self._buffer[startpos + 1 : le])
                    except ValueError as exc:
                        raise RedisProtocolException(str(exc))
                    startpos = le + 2
                    if arraysize < 0:
                        yield None
                    else:
                        array = []
                        if arraysize:
                            p = self._parser_gen(startpos)
                            try:
                                while arraysize:
                                    nv = next(p)
                                    if nv is not False:
                                        array.append(nv)
                                        arraysize -= 1
                                    else:
                                        yield False
                            finally:
                                p.close()
                                startpos = self._lastpos
                        yield array
                else:
                    raise RedisProtocolException(repr(c) + ' is not a valid RESP type')
        finally:
            self._lastpos = startpos

@defaultconfig
class Redis(Protocol):
    '''
    Redis (RESP) Protocol
    '''
    _default_persist = True
    # Default Redis server port
    _default_defaultport = 6379
    _default_createqueue = True
    # Limit response messages queue size
    _default_messagequeuesize = 4096
    # Default limit Redis bulk string to 64MB
    _default_bulklimit = 67108864
    # Default limit Redis array level to 128 levels
    _default_levellimit = 128
    # Default encoding when using Unicode strings in Redis commands
    _default_encoding = 'utf-8'
    # Use hiredis if possible
    _default_hiredis = True
    # Send PING command when the connection is idle
    _default_keepalivetime = 10
    # Disconnect when PING command does not have response
    _default_keepalivetimeout = 5
    # Connect timeout
    _default_connect_timeout = 5
    _default_tcp_nodelay = True
    _logger = logging.getLogger(__name__ + '.Redis')
    def __init__(self):
        '''
        Constructor
        '''
        Protocol.__init__(self)
        self.usehiredis = hiredis_available and self.hiredis
        self._format_request_init(self.encoding)
    def init(self, connection):
        for m in Protocol.init(self, connection):
            yield m
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority - 1, RedisConnectionStateEvent.createMatcher(connection = connection), ('connstate', connection)))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority + 1, RedisResponseEvent.createMatcher(connection = connection), ('response', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority + 1, RedisSubscribeEvent.createMatcher(connection = connection), ('subscribe', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, RedisSubscribeMessageEvent.createMatcher(connection = connection), ('message', connection), self.messagequeuesize))
        connection.redis_subscribe = False
        connection.redis_select = None
        connection.redis_subscribe_keys = set()
        connection.redis_subscribe_pkeys = set()
        for m in self.reconnect_init(connection):
            yield m
    def _format_request_init(self, encoding):
        def _bytes(a):
            if isinstance(a, bytes):
                return a
            else:
                return str(a).encode(encoding)
        def format_request(*args):
            return b'*' + str(len(args)).encode('ascii') + b'\r\n' + b''.join(itertools.chain.from_iterable((b'$',str(len(sa)).encode('ascii'),b'\r\n',sa,b'\r\n') for sa in (_bytes(a) for a in args)))
        self._bytes = _bytes
        self.format_request = format_request
    def reconnect_init(self, connection):
        connection.xid = 1
        connection.redis_replyxid = 1
        connection.redis_ping = -1
        connection.redis_pingreply = -1
        connection.redis_bufferedxid = 0
        connection.redis_sendbuffer = []
        connection.redis_sender = False
        connection.redis_locker = object()
        write_buffer = []
        if self.usehiredis:
            connection.redis_reader = hiredis.Reader(protocolError = RedisProtocolException, replyError = RedisReplyException)
        else:
            connection.redis_reader = RedisParser()
        if connection.redis_select:
            write_buffer.append(self.format_request(b'SELECT', connection.redis_select))
            connection.xid += 1
        if connection.redis_subscribe:
            if connection.redis_subscribe_keys:
                write_buffer.append(self.format_request(b'SUBSCRIBE', *tuple(connection.redis_subscribe_keys)))
            if connection.redis_subscribe_pkeys:
                write_buffer.append(self.format_request(b'PSUBSCRIBE', *tuple(connection.redis_subscribe_pkeys)))
        connection.scheduler.emergesend(ConnectionWriteEvent(connection, connection.connmark, data=b''.join(write_buffer)))
        for m in connection.waitForSend(RedisConnectionStateEvent(RedisConnectionStateEvent.CONNECTION_UP, connection, connection.connmark, self)):
            yield m
    def closed(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        self._logger.info('Redis connection is closed on %r', connection)
        for m in connection.waitForSend(RedisConnectionStateEvent(RedisConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
            yield m
    def error(self, connection):
        for m in Protocol.error(self, connection):
            yield m
        self._logger.warning('Redis connection is reset on %r', connection)
        for m in connection.waitForSend(RedisConnectionStateEvent(RedisConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
            yield m
    def replymatcher(self, requestid, connection, iserror = None):
        """
        Create an event matcher to match 
        """
        matcherparam = [connection, connection.connmark, requestid]
        if iserror is not None:
            matcherparam.append(iserror)
        return RedisResponseEvent.createMatcher(*matcherparam)
    def subscribematcher(self, connection, subscribe = None, channel = None, type = None):
        return RedisSubscribeMessageEvent.createMatcher(type, subscribe, channel, connection)
    def statematcher(self, connection, state = RedisConnectionStateEvent.CONNECTION_DOWN, currentconn = True):
        if currentconn:
            return RedisConnectionStateEvent.createMatcher(state, connection, connection.connmark)
        else:
            return RedisConnectionStateEvent.createMatcher(state, connection)

    def _prepare_command(self, connection, args):
        cmdname = _str(args[0]).upper()
        if cmdname in ('SUBSCRIBE', 'UNSUBSCRIBE', 'PSUBSCRIBE', 'PUNSUBSCRIBE'):
            if len(args) <= 1:
                raise RedisProtocolException('(P)SUBSCRIBE/(P)UNSUBSCRIBE must have at least one key as parameter')
            connection.redis_subscribe = True
            if cmdname == 'SUBSCRIBE':
                connection.redis_subscribe_keys.update(self._bytes(a) for a in args[1:])
            elif cmdname == 'UNSUBSCRIBE':
                connection.redis_subscribe_keys.difference_update(self._bytes(a) for a in args[1:])
            elif cmdname == 'PSUBSCRIBE':
                connection.redis_subscribe_pkeys.update(self._bytes(a) for a in args[1:])
            elif cmdname == 'PUNSUBSCRIBE':
                connection.redis_subscribe_pkeys.difference_update(self._bytes(a) for a in args[1:])
            reply_matcher = RedisSubscribeEvent.createMatcher(cmdname.lower(), args[-1], connection, connection.connmark)
        elif connection.redis_subscribe_keys or connection.redis_subscribe_pkeys:
            if cmdname == 'PING':
                rid = connection.redis_ping
                connection.redis_ping -= 1
                reply_matcher = RedisResponseEvent.createMatcher(connection, connection.connmark, rid)
            elif cmdname == 'QUIT':
                rid = 0
                reply_matcher = RedisResponseEvent.createMatcher(connection, connection.connmark, rid)
                connection.need_reconnect = False
            else:
                raise RedisProtocolException('Can only use PING/QUIT/(P)SUBSCRIBE/(P)UNSUBSCRIBE on a subscribed connection')
        else:
            if cmdname == 'SELECT':
                if len(args) != 2:
                    raise RedisProtocolException("wrong number of arguments for 'select' command")
                connection.redis_select = _str(args[1])
            elif cmdname == 'QUIT':
                connection.need_reconnect = False
            rid = connection.xid
            connection.xid += 1
            reply_matcher = RedisResponseEvent.createMatcher(connection, connection.connmark, rid)
        r = self.format_request(*args)
        return r, reply_matcher
    def send_command(self, connection, container, *args):
        '''
        Send command to Redis server.
        
        :param connection: Redis connection
        
        :param container: routine container
        
        :param \*args: command paramters, begin with command name, e.g. `'SET'`,`'key'`,`'value'`
        
        :returns: Event matcher to wait for reply. The value is returned from container.retvalue
        '''
        with closing(container.delegateOther(self._send_command(connection, container, *args),
                                             container, forceclose = True)) as g:
            for m in g:
                yield m
    def _send_command(self, connection, container, *args):
        if not args:
            raise RedisProtocolException('No command name')
        l = Lock(connection.redis_locker, connection.scheduler)
        # The socket write sequence must be the same as the send sequence, add a lock to ensure that
        for m in l.lock(container):
            yield m
        with l:
            r, reply_matcher = self._prepare_command(connection, args)
            for m in connection.write(ConnectionWriteEvent(connection, connection.connmark, data = r), False):
                yield m
        container.retvalue = reply_matcher
    def send_batch(self, connection, container, *cmds):
        '''
        Send multiple commands to redis server at once
        
        :param connection: redis connection
        
        :param container: routine container
        
        :param \*cmds: commands to send. Each command is a tuple/list of bytes/str.
        
        :returns: list of reply event matchers (from container.retvalue)
        '''
        with closing(container.delegateOther(self._send_batch(connection, container, *cmds),
                                         container, forceclose = True)) as g:
            for m in g:
                yield m
    def _send_batch(self, connection, container, *cmds):
        "Use delegate to ensure it always ends"
        if not cmds:
            raise RedisProtocolException('No commands')
        l = Lock(connection.redis_locker, connection.scheduler)
        for m in l.lock(container):
            yield m
        with l:
            commands = []
            matchers = []
            for c in cmds:
                try:
                    r, reply_matcher = self._prepare_command(connection, c)
                    commands.append(r)
                    matchers.append(reply_matcher)
                except:
                    self._logger.warning('Error in one of the commands in a batch: %r. The command is ignored.', c, exc_info = True)
            if not commands:
                raise RedisProtocolException('Error for every command in a batch')
            for m in connection.write(ConnectionWriteEvent(connection, connection.connmark, data = b''.join(commands)), False):
                yield m
        container.retvalue = matchers
    def execute_command(self, connection, container, *args):
        '''
        Send command to Redis server and wait for response
        
        :param connection: Redis connection
        
        :param container: routine container
        
        :param \*args: command paramters, begin with command name, e.g. `'SET'`,`'key'`,`'value'`
        
        :returns: Response from Redis server. The value is returned from container.retvalue
        
        :raises RedisReplyException: Redis server returns an error (e.g. "-ERR ...")
        '''
        for m in self.send_command(connection, container, *args):
            yield m
        rm = container.retvalue
        sm = self.statematcher(connection)
        yield (sm, rm)
        if container.matcher is sm:
            raise RedisProtocolException('Redis connection down before response received')
        else:
            r = container.event.result
            if isinstance(r, Exception):
                raise r
            else:
                container.retvalue = r
    def batch_execute(self, connection, container, *cmds):
        '''
        Send multiple commands to redis server at once, and get responses
        
        :param connection: redis connection
        
        :param container: routine container
        
        :param \*cmds: commands to send. Each command is a tuple/list of bytes/str.
        
        :returns: list of replies (from container.retvalue). Exceptions are NOT raised.
        '''
        if not cmds:
            container.retvalue = []
            return
        for m in self.send_batch(connection, container, *cmds):
            yield m
        matchers = container.retvalue
        sm = self.statematcher(connection)
        retvalue = []
        for m in matchers:
            yield (m, sm)
            if container.matcher is sm:
                raise RedisProtocolException('Redis connection down before response received')
            retvalue.append(container.event.result)
        container.retvalue = retvalue
    def parse(self, connection, data, laststart):
        events = []
        connection.redis_reader.feed(_copy(data))
        while True:
            r = connection.redis_reader.gets()
            if r is False:
                break
            if connection.redis_replyxid < connection.xid:
                events.append(RedisResponseEvent(connection, connection.connmark, connection.redis_replyxid, isinstance(r, Exception), self, result = r))
                connection.redis_replyxid += 1
            elif connection.redis_subscribe:
                if isinstance(r, bytes) or isinstance(r, Exception):
                    events.append(RedisResponseEvent(connection, connection.connmark, 0, isinstance(r, Exception), self, result = r))
                elif r[0] == b'message':
                    events.append(RedisSubscribeMessageEvent(RedisSubscribeMessageEvent.MESSAGE, r[1], r[1],
                                                             connection, connection.connmark, self, message = r[2]))
                elif r[0] == b'pmessage':
                    events.append(RedisSubscribeMessageEvent(RedisSubscribeMessageEvent.PMESSAGE, r[1], r[2],
                                                             connection, connection.connmark, self, message = r[3]))
                elif r[0] == b'pong':
                    events.append(RedisResponseEvent(connection, connection.connmark, connection.redis_pingreply, False, self, result = r[1]))
                    connection.redis_pingreply -= 1
                else:
                    events.append(RedisSubscribeEvent(_str(r[0]), r[1], connection, connection.connmark, self, result = r))
                    if not r[2]:
                        connection.redis_subscribe = False
                        connection.xid += connection.redis_bufferedxid
        if laststart == len(data):
            # Remote write close
            events.append(ConnectionWriteEvent(connection, connection.connmark, data = b'', EOF = True))
        return (events, 0)
    def notconnected(self, connection):
        for m in Protocol.notconnected(self, connection):
            yield m
        for m in connection.waitForSend(RedisConnectionStateEvent(RedisConnectionStateEvent.CONNECTION_NOTCONNECTED,
                                                                  connection, connection.connmark, self)):
            yield m
    def keepalive(self, connection):
        try:
            if connection.redis_replyxid == connection.xid:
                for m in connection.executeWithTimeout(self.keepalivetimeout, self.execute_command(connection, connection, 'PING')):
                    yield m
                if connection.timeout:
                    for m in connection.reset(True):
                        yield m
        except Exception:
            for m in connection.reset(True):
                yield m
    @staticmethod
    def reconnect_timeseq():
        yield 0
        nextSeq = 0.5
        while True:
            yield nextSeq                
            if nextSeq < 16:
                nextSeq = nextSeq * 2
            else:
                nextSeq = 20
        


'''
Created on 2015/11/26

:author: hubo
'''

import zlib
import codecs
from .gzipheader import header, tail
import time

def unicode_encoder(encoding, errors = 'strict'):
    return codecs.getincrementalencoder(encoding)(errors).encode

def unicode_decoder(encoding, errors = 'strict'):
    return codecs.getincrementaldecoder(encoding)(errors).decode

def donothing_encoder(x, iseof):
    return x

if str is bytes:
    def str_encoder(encoding, errors = 'strict'):
        return donothing_encoder
    def str_decoder(encoding, errors = 'strict'):
        return donothing_encoder
else:
    str_encoder = unicode_encoder
    str_decoder = unicode_decoder

def deflate_encoder(level = None):
    if level is None:
        obj = zlib.compressobj()
    else:
        obj = zlib.compressobj(level)
    def enc(data, final):
        ret = obj.compress(data)
        if final:
            ret += obj.flush()
        return ret
    return enc

def deflate_decoder(wbits = None):
    if wbits is None:
        obj = zlib.decompressobj()
    else:
        obj = zlib.decompressobj(wbits)
    def enc(data, final):
        ret = obj.decompress(data)
        if final:
            ret += obj.flush()
        return ret
    return enc

def _tobytes(s, encoding = 'utf-8'):
    if s is bytes:
        return s
    else:
        return s.encode(encoding)

class GzipEncoder(object):
    def __init__(self, fname = 'tmp', level = 9):
        self.crc = 0
        self.size = 0
        self.fname = _tobytes(fname)
        self.level = level
        self.writeheader = False
    def enc(self, data, final):
        buf = []
        if not self.writeheader:
            h = header.new()
            h.mtime = int(time.time())
            h.fname = self.fname
            buf.append(header.tobytes(h))
            self.compobj = zlib.compressobj(self.level, zlib.DEFLATED, -zlib.MAX_WBITS, zlib.DEF_MEM_LEVEL)
            self.writeheader = True
        buf.append(self.compobj.compress(data))
        self.crc = zlib.crc32(data, self.crc) & 0xffffffff
        self.size += len(data)
        if final:
            buf.append(self.compobj.flush())
            t = tail.new()
            t.crc32 = self.crc
            t.isize = self.size
            buf.append(tail.tobytes(t))
        return b''.join(buf)
        
def gzip_encoder(fname = 'tmp', level = 9):
    return GzipEncoder(fname, level).enc

class GzipDecoder(object):
    def __init__(self):
        self.crc = 0
        self.size = 0
        self.readheader = True
        self.readtail = False
        self.buffer = b''
    def enc(self, data, final):
        buf = []
        self.buffer += data
        while True:
            if self.readheader:
                r = header.parse(self.buffer)
                if r is None:
                    break
                h, size = r
                if h.id1 != 0x1f or h.id2 != 0x8b or h.cm != 8:
                    raise ValueError('Unsupported format')
                self.decompobj = zlib.decompressobj(-zlib.MAX_WBITS)
                self.buffer = self.buffer[size:]
                self.readheader = False
            elif self.readtail:
                r = tail.parse(self.buffer)
                if r is None:
                    break
                t, size = r
                if t.crc32 != self.crc or t.isize != self.size:
                    raise ValueError('Checksum not met')
                self.crc = 0
                self.size = 0
                self.buffer = self.buffer[size:]
                self.readtail = False
                self.readheader = True
            else:
                newdata = self.decompobj.decompress(self.buffer)
                self.crc = zlib.crc32(newdata, self.crc) & 0xFFFFFFFF
                self.size += len(newdata)
                buf.append(newdata)
                if self.decompobj.unused_data:
                    self.buffer = self.decompobj.unused_data
                    newdata = self.decompobj.flush()
                    self.crc = zlib.crc32(newdata, self.crc) & 0xFFFFFFFF
                    self.size += len(newdata)
                    buf.append(newdata)
                    self.readtail = True
                else:
                    self.buffer = b''
                    break
        if final:
            if self.buffer or not self.readheader:
                raise ValueError('Unexpected EOF')
        return b''.join(buf)

def gzip_decoder():
    return GzipDecoder().enc



'''
Created on 2016/2/26

:author: think
'''


from vlcp.config import defaultconfig
from vlcp.server.module import Module, api, depend, callAPI, ModuleNotification
from vlcp.event.runnable import RoutineContainer
from vlcp.service.sdn import ovsdbmanager
from vlcp.event.connection import ConnectionResetException
from vlcp.event.event import Event, withIndices
from vlcp.service.sdn.ovsdbmanager import OVSDBBridgeSetup, OVSDBConnectionSetup,\
    OVSDBBridgeNotAppearException
import vlcp.utils.ovsdb as ovsdb
from vlcp.protocol.jsonrpc import JsonRPCErrorResultException,\
    JsonRPCNotificationEvent, JsonRPCProtocolException
import itertools
from contextlib import closing

def _bytes(s):
    return s.encode('ascii')

@withIndices('connection')
class OVSDBConnectionPortsSynchronized(Event):
    pass

@withIndices('connection', 'name', 'ofport', 'id', 'vhost', 'datapathid')
class OVSDBPortUpNotification(Event):
    pass

class OVSDBPortNotAppearException(Exception):
    pass


@defaultconfig
@depend(ovsdbmanager.OVSDBManager)
class OVSDBPortManager(Module):
    '''
    Manage Ports from OVSDB Protocol
    '''
    service = True
    def __init__(self, server):
        Module.__init__(self, server)
        self.apiroutine = RoutineContainer(self.scheduler)
        self.apiroutine.main = self._manage_ports
        self.routines.append(self.apiroutine)
        self.managed_ports = {}
        self.managed_ids = {}
        self.monitor_routines = set()
        self.ports_uuids = {}
        self.wait_portnos = {}
        self.wait_names = {}
        self.wait_ids = {}
        self.bridge_datapathid = {}
        self.createAPI(api(self.getports, self.apiroutine),
                       api(self.getallports, self.apiroutine),
                       api(self.getportbyid, self.apiroutine),
                       api(self.waitportbyid, self.apiroutine),
                       api(self.getportbyname, self.apiroutine),
                       api(self.waitportbyname, self.apiroutine),
                       api(self.getportbyno, self.apiroutine),
                       api(self.waitportbyno, self.apiroutine),
                       api(self.resync, self.apiroutine)
                       )
        self._synchronized = False
    def _get_interface_info(self, connection, protocol, buuid, interface_uuid, port_uuid):
        try:
            method, params = ovsdb.transact('Open_vSwitch',
                                            ovsdb.wait('Interface', [["_uuid", "==", ovsdb.uuid(interface_uuid)]],
                                                       ["ofport"], [{"ofport":ovsdb.oset()}], False, 5000),
                                            ovsdb.wait('Interface', [["_uuid", "==", ovsdb.uuid(interface_uuid)]],
                                                       ["ifindex"], [{"ifindex":ovsdb.oset()}], False, 5000),
                                            ovsdb.select('Interface', [["_uuid", "==", ovsdb.uuid(interface_uuid)]],
                                                                         ["_uuid", "name", "ifindex", "ofport", "type", "external_ids"]))
            for m in protocol.querywithreply(method, params, connection, self.apiroutine):
                yield m
            r = self.apiroutine.jsonrpc_result[0]
            if 'error' in r:
                raise JsonRPCErrorResultException('Error while acquiring interface: ' + repr(r['error']))            
            r = self.apiroutine.jsonrpc_result[1]
            if 'error' in r:
                raise JsonRPCErrorResultException('Error while acquiring interface: ' + repr(r['error']))            
            r = self.apiroutine.jsonrpc_result[2]
            if 'error' in r:
                raise JsonRPCErrorResultException('Error while acquiring interface: ' + repr(r['error']))
            if not r['rows']:
                self.apiroutine.retvalue = []
                return
            r0 = r['rows'][0]
            if r0['ofport'] < 0:
                # Ignore this port because it is in an error state
                self.apiroutine.retvalue = []
                return
            r0['_uuid'] = r0['_uuid'][1]
            r0['ifindex'] = ovsdb.getoptional(r0['ifindex'])
            r0['external_ids'] = ovsdb.getdict(r0['external_ids'])
            if buuid not in self.bridge_datapathid:
                self.apiroutine.retvalue = []
                return
            else:
                datapath_id = self.bridge_datapathid[buuid]
            if 'iface-id' in r0['external_ids']:
                eid = r0['external_ids']['iface-id']
                r0['id'] = eid
                id_ports = self.managed_ids.setdefault((protocol.vhost, eid), [])
                id_ports.append((datapath_id, r0))
            else:
                r0['id'] = None
            self.managed_ports.setdefault((protocol.vhost, datapath_id),[]).append((port_uuid, r0))
            notify = False
            if (protocol.vhost, datapath_id, r0['ofport']) in self.wait_portnos:
                notify = True
                del self.wait_portnos[(protocol.vhost, datapath_id, r0['ofport'])]
            if (protocol.vhost, datapath_id, r0['name']) in self.wait_names:
                notify = True
                del self.wait_names[(protocol.vhost, datapath_id, r0['name'])]
            if (protocol.vhost, r0['id']) in self.wait_ids:
                notify = True
                del self.wait_ids[(protocol.vhost, r0['id'])]
            if notify:
                for m in self.apiroutine.waitForSend(OVSDBPortUpNotification(connection, r0['name'],
                                                                             r0['ofport'], r0['id'],
                                                                             protocol.vhost, datapath_id,
                                                                             port = r0)):
                    yield m
            self.apiroutine.retvalue = [r0]
        except JsonRPCProtocolException:
            self.apiroutine.retvalue = []
    def _remove_interface_id(self, connection, protocol, datapath_id, port):
        eid = port['id']
        eid_list = self.managed_ids.get((protocol.vhost, eid))
        for i in range(0, len(eid_list)):
            if eid_list[i][1]['_uuid'] == port['_uuid']:
                del eid_list[i]
                break
    def _remove_interface(self, connection, protocol, datapath_id, interface_uuid, port_uuid):
        ports = self.managed_ports.get((protocol.vhost, datapath_id))
        r = None
        if ports is not None:
            for i in range(0, len(ports)):
                if ports[i][1]['_uuid'] == interface_uuid:
                    r = ports[i][1]
                    if r['id']:
                        self._remove_interface_id(connection, protocol, datapath_id, r)
                    del ports[i]
                    break
            if not ports:
                del self.managed_ports[(protocol.vhost, datapath_id)]
        return r
    def _remove_all_interface(self, connection, protocol, datapath_id, port_uuid, buuid):
        ports = self.managed_ports.get((protocol.vhost, datapath_id))
        if ports is not None:
            removed_ports = [r for puuid, r in ports if puuid == port_uuid]
            not_removed_ports = [(puuid, r) for puuid, r in ports if puuid != port_uuid]
            ports[:len(not_removed_ports)] = not_removed_ports
            del ports[len(not_removed_ports):]
            for r in removed_ports:
                if r['id']:
                    self._remove_interface_id(connection, protocol, datapath_id, r)
            if not ports:
                del self.managed_ports[(protocol.vhost, datapath_id)]
            return removed_ports
        if port_uuid in self.ports_uuids and self.ports_uuids[port_uuid] == buuid:
            del self.ports_uuids[port_uuid]
        return []
    def _update_interfaces(self, connection, protocol, updateinfo, update = True):
        """
        There are several kinds of updates, they may appear together:
        
        1. New bridge created (or from initial updateinfo). We should add all the interfaces to the list.
        
        2. Bridge removed. Remove all the ports.
        
        3. name and datapath_id may be changed. We will consider this as a new bridge created, and an old
           bridge removed.
        
        4. Bridge ports modification, i.e. add/remove ports.
           a) Normally a port record is created/deleted together. A port record cannot exist without a
              bridge containing it.
               
           b) It is also possible that a port is removed from one bridge and added to another bridge, in
              this case the ports do not appear in the updateinfo
            
        5. Port interfaces modification, i.e. add/remove interfaces. The bridge record may not appear in this
           situation.
           
        We must consider these situations carefully and process them in correct order.
        """
        port_update = updateinfo.get('Port', {})
        bridge_update = updateinfo.get('Bridge', {})
        working_routines = []
        def process_bridge(buuid, uo):
            try:
                nv = uo['new']
                if 'datapath_id' in nv:
                    if ovsdb.getoptional(nv['datapath_id']) is None:
                        # This bridge is not initialized. Wait for the bridge to be initialized.
                        for m in callAPI(self.apiroutine, 'ovsdbmanager', 'waitbridge', {'connection': connection,
                                                                        'name': nv['name'],
                                                                        'timeout': 5}):
                            yield m
                        datapath_id = self.apiroutine.retvalue
                    else:
                        datapath_id = int(nv['datapath_id'], 16)
                    self.bridge_datapathid[buuid] = datapath_id
                elif buuid in self.bridge_datapathid:
                    datapath_id = self.bridge_datapathid[buuid]
                else:
                    # This should not happen, but just in case...
                    for m in callAPI(self.apiroutine, 'ovsdbmanager', 'waitbridge', {'connection': connection,
                                                                    'name': nv['name'],
                                                                    'timeout': 5}):
                        yield m
                    datapath_id = self.apiroutine.retvalue
                    self.bridge_datapathid[buuid] = datapath_id
                if 'ports' in nv:
                    nset = set((p for _,p in ovsdb.getlist(nv['ports'])))
                else:
                    nset = set()
                if 'old' in uo:
                    ov = uo['old']
                    if 'ports' in ov:
                        oset = set((p for _,p in ovsdb.getlist(ov['ports'])))
                    else:
                        # new ports are not really added; it is only sent because datapath_id is modified
                        nset = set()
                        oset = set()
                    if 'datapath_id' in ov and ovsdb.getoptional(ov['datapath_id']) is not None:
                        old_datapathid = int(ov['datapath_id'], 16)
                    else:
                        old_datapathid = datapath_id
                else:
                    oset = set()
                    old_datapathid = datapath_id
                # For every deleted port, remove the interfaces with this port _uuid
                remove = []
                add_routine = []
                for puuid in oset - nset:
                    remove += self._remove_all_interface(connection, protocol, old_datapathid, puuid, buuid)
                # For every port not changed, check if the interfaces are modified;
                for puuid in oset.intersection(nset):
                    if puuid in port_update:
                        # The port is modified, there should be an 'old' set and 'new' set
                        pu = port_update[puuid]
                        if 'old' in pu:
                            poset = set((p for _,p in ovsdb.getlist(pu['old']['interfaces'])))
                        else:
                            poset = set()
                        if 'new' in pu:
                            pnset = set((p for _,p in ovsdb.getlist(pu['new']['interfaces'])))
                        else:
                            pnset = set()
                        # Remove old interfaces
                        remove += [r for r in 
                                   (self._remove_interface(connection, protocol, datapath_id, iuuid, puuid)
                                    for iuuid in (poset - pnset)) if r is not None]
                        # Prepare to add new interfaces
                        add_routine += [self._get_interface_info(connection, protocol, buuid, iuuid, puuid)
                                        for iuuid in (pnset - poset)]
                # For every port added, add the interfaces
                def add_port_interfaces(puuid):
                    # If the uuid does not appear in update info, we have no choice but to query interfaces with select
                    # we cannot use data from other bridges; the port may be moved from a bridge which is not tracked
                    try:
                        method, params = ovsdb.transact('Open_vSwitch', ovsdb.select('Port',
                                                                                     [["_uuid", "==", ovsdb.uuid(puuid)]],
                                                                                     ["interfaces"]))
                        for m in protocol.querywithreply(method, params, connection, self.apiroutine):
                            yield m
                        r = self.apiroutine.jsonrpc_result[0]
                        if 'error' in r:
                            raise JsonRPCErrorResultException('Error when query interfaces from port ' + repr(puuid) + ': ' + r['error'])
                        if r['rows']:
                            interfaces = ovsdb.getlist(r['rows'][0]['interfaces'])
                            with closing(self.apiroutine.executeAll([self._get_interface_info(connection, protocol, buuid, iuuid, puuid)
                                                                 for _,iuuid in interfaces])) as g:
                                for m in g:
                                    yield m
                            self.apiroutine.retvalue = list(itertools.chain(r[0] for r in self.apiroutine.retvalue))
                        else:
                            self.apiroutine.retvalue = []
                    except JsonRPCProtocolException:
                        self.apiroutine.retvalue = []
                    except ConnectionResetException:
                        self.apiroutine.retvalue = []
                
                for puuid in nset - oset:
                    self.ports_uuids[puuid] = buuid
                    if puuid in port_update and 'new' in port_update[puuid] \
                            and 'old' not in port_update[puuid]:
                        # Add all the interfaces in 'new'
                        interfaces = ovsdb.getlist(port_update[puuid]['new']['interfaces'])
                        add_routine += [self._get_interface_info(connection, protocol, buuid, iuuid, puuid)
                                        for _,iuuid in interfaces]
                    else:
                        add_routine.append(add_port_interfaces(puuid))
                # Execute the add_routine
                try:
                    with closing(self.apiroutine.executeAll(add_routine)) as g:
                        for m in g:
                            yield m
                except:
                    add = []
                    raise
                else:
                    add = list(itertools.chain(r[0] for r in self.apiroutine.retvalue))
                finally:
                    if update:
                        self.scheduler.emergesend(
                                ModuleNotification(self.getServiceName(), 'update',
                                                   datapathid = datapath_id,
                                                   connection = connection,
                                                   vhost = protocol.vhost,
                                                   add = add, remove = remove,
                                                   reason = 'bridgemodify'
                                                            if 'old' in uo
                                                            else 'bridgeup'
                                                   ))
            except JsonRPCProtocolException:
                pass
            except ConnectionResetException:
                pass
            except OVSDBBridgeNotAppearException:
                pass
        ignore_ports = set()
        for buuid, uo in bridge_update.items():
            # Bridge removals are ignored because we process OVSDBBridgeSetup event instead
            if 'old' in uo:
                if 'ports' in uo['old']:
                    oset = set((puuid for _, puuid in ovsdb.getlist(uo['old']['ports'])))
                    ignore_ports.update(oset)
                if 'new' not in uo:
                    if buuid in self.bridge_datapathid:
                        del self.bridge_datapathid[buuid]
            if 'new' in uo:
                # If bridge contains this port is updated, we process the port update totally in bridge,
                # so we ignore it later
                if 'ports' in uo['new']:
                    nset = set((puuid for _, puuid in ovsdb.getlist(uo['new']['ports'])))
                    ignore_ports.update(nset)
                working_routines.append(process_bridge(buuid, uo))                
        def process_port(buuid, port_uuid, interfaces, remove_ids):
            if buuid not in self.bridge_datapathid:
                return
            datapath_id = self.bridge_datapathid[buuid]
            ports = self.managed_ports.get((protocol.vhost, datapath_id))
            remove = []
            if ports is not None:
                remove = [p for _,p in ports if p['_uuid'] in remove_ids]
                not_remove = [(_,p) for _,p in ports if p['_uuid'] not in remove_ids]
                ports[:len(not_remove)] = not_remove
                del ports[len(not_remove):]
            if interfaces:
                try:
                    with closing(self.apiroutine.executeAll([self._get_interface_info(connection, protocol, buuid, iuuid, port_uuid)
                                                         for iuuid in interfaces])) as g:
                        for m in g:
                            yield m
                    add = list(itertools.chain((r[0] for r in self.apiroutine.retvalue if r[0])))
                except Exception:
                    self._logger.warning("Cannot get new port information", exc_info = True)
                    add = []
            else:
                add = []
            if update:
                for m in self.apiroutine.waitForSend(ModuleNotification(self.getServiceName(), 'update', datapathid = datapath_id,
                                                                                                          connection = connection,
                                                                                                          vhost = protocol.vhost,
                                                                                                          add = add, remove = remove,
                                                                                                          reason = 'bridgemodify'
                                                                                                          )):
                    yield m
        for puuid, po in port_update.items():
            if puuid not in ignore_ports:
                bridge_id = self.ports_uuids.get(puuid)
                if bridge_id is not None:
                    datapath_id = self.bridge_datapathid[bridge_id]
                    if datapath_id is not None:
                        # This port is modified
                        if 'new' in po:
                            nset = set((iuuid for _, iuuid in ovsdb.getlist(po['new']['interfaces'])))
                        else:
                            nset = set()                    
                        if 'old' in po:
                            oset = set((iuuid for _, iuuid in ovsdb.getlist(po['old']['interfaces'])))
                        else:
                            oset = set()
                        working_routines.append(process_port(bridge_id, puuid, nset - oset, oset - nset))
        if update:
            for r in working_routines:
                self.apiroutine.subroutine(r)
        else:
            try:
                with closing(self.apiroutine.executeAll(working_routines, None, ())) as g:
                    for m in g:
                        yield m
            finally:
                self.scheduler.emergesend(OVSDBConnectionPortsSynchronized(connection))
    def _get_ports(self, connection, protocol):
        try:
            try:
                method, params = ovsdb.monitor('Open_vSwitch', 'ovsdb_port_manager_interfaces_monitor', {
                                                    'Bridge':[ovsdb.monitor_request(["name", "datapath_id", "ports"])],
                                                    'Port':[ovsdb.monitor_request(["interfaces"])]
                                                })
                try:
                    for m in protocol.querywithreply(method, params, connection, self.apiroutine):
                        yield m
                except JsonRPCErrorResultException:
                    # The monitor is already set, cancel it first
                    method2, params2 = ovsdb.monitor_cancel('ovsdb_port_manager_interfaces_monitor')
                    for m in protocol.querywithreply(method2, params2, connection, self.apiroutine, False):
                        yield m
                    for m in protocol.querywithreply(method, params, connection, self.apiroutine):
                        yield m
                r = self.apiroutine.jsonrpc_result
            except:
                for m in self.apiroutine.waitForSend(OVSDBConnectionPortsSynchronized(connection)):
                    yield m
                raise
            # This is the initial state, it should contains all the ids of ports and interfaces
            self.apiroutine.subroutine(self._update_interfaces(connection, protocol, r, False))
            update_matcher = JsonRPCNotificationEvent.createMatcher('update', connection, connection.connmark,
                                                                    _ismatch = lambda x: x.params[0] == 'ovsdb_port_manager_interfaces_monitor')
            conn_state = protocol.statematcher(connection)
            while True:
                yield (update_matcher, conn_state)
                if self.apiroutine.matcher is conn_state:
                    break
                else:
                    self.apiroutine.subroutine(self._update_interfaces(connection, protocol, self.apiroutine.event.params[1], True))
        except JsonRPCProtocolException:
            pass
        finally:
            if self.apiroutine.currentroutine in self.monitor_routines:
                self.monitor_routines.remove(self.apiroutine.currentroutine)
    def _get_existing_ports(self):
        for m in callAPI(self.apiroutine, 'ovsdbmanager', 'getallconnections', {'vhost':None}):
            yield m
        matchers = []
        for c in self.apiroutine.retvalue:
            self.monitor_routines.add(self.apiroutine.subroutine(self._get_ports(c, c.protocol)))
            matchers.append(OVSDBConnectionPortsSynchronized.createMatcher(c))
        for m in self.apiroutine.waitForAll(*matchers):
            yield m
        self._synchronized = True
        for m in self.apiroutine.waitForSend(ModuleNotification(self.getServiceName(), 'synchronized')):
            yield m
    def _wait_for_sync(self):
        if not self._synchronized:
            yield (ModuleNotification.createMatcher(self.getServiceName(), 'synchronized'),)
    def _manage_ports(self):
        try:
            self.apiroutine.subroutine(self._get_existing_ports())
            connsetup = OVSDBConnectionSetup.createMatcher()
            bridgedown = OVSDBBridgeSetup.createMatcher(OVSDBBridgeSetup.DOWN)
            while True:
                yield (connsetup, bridgedown)
                e = self.apiroutine.event
                if self.apiroutine.matcher is connsetup:
                    self.monitor_routines.add(self.apiroutine.subroutine(self._get_ports(e.connection, e.connection.protocol)))
                else:
                    # Remove ports of the bridge
                    ports =  self.managed_ports.get((e.vhost, e.datapathid))
                    if ports is not None:
                        ports_original = ports
                        ports = [p for _,p in ports]
                        for p in ports:
                            if p['id']:
                                self._remove_interface_id(e.connection,
                                                          e.connection.protocol, e.datapathid, p)
                        newdpid = getattr(e, 'new_datapath_id', None)
                        buuid = e.bridgeuuid
                        if newdpid is not None:
                            # This bridge changes its datapath id
                            if buuid in self.bridge_datapathid and self.bridge_datapathid[buuid] == e.datapathid:
                                self.bridge_datapathid[buuid] = newdpid
                            def re_add_interfaces():
                                with closing(self.apiroutine.executeAll(
                                    [self._get_interface_info(e.connection, e.connection.protocol, buuid,
                                                              r['_uuid'], puuid)
                                     for puuid, r in ports_original])) as g:
                                    for m in g:
                                        yield m
                                add = list(itertools.chain(r[0] for r in self.apiroutine.retvalue))
                                for m in self.apiroutine.waitForSend(ModuleNotification(self.getServiceName(),
                                                  'update', datapathid = e.datapathid,
                                                  connection = e.connection,
                                                  vhost = e.vhost,
                                                  add = add, remove = [],
                                                  reason = 'bridgeup'
                                                  )):
                                    yield m
                            self.apiroutine.subroutine(re_add_interfaces())
                        else:
                            # The ports are removed
                            for puuid, _ in ports_original:
                                if puuid in self.ports_uuids[puuid] and self.ports_uuids[puuid] == buuid:
                                    del self.ports_uuids[puuid]
                        del self.managed_ports[(e.vhost, e.datapathid)]
                        self.scheduler.emergesend(ModuleNotification(self.getServiceName(),
                                                  'update', datapathid = e.datapathid,
                                                  connection = e.connection,
                                                  vhost = e.vhost,
                                                  add = [], remove = ports,
                                                  reason = 'bridgedown'
                                                  ))                            
        finally:
            for r in list(self.monitor_routines):
                r.close()
            self.scheduler.emergesend(ModuleNotification(self.getServiceName(), 'unsynchronized'))
    def getports(self, datapathid, vhost = ''):
        "Return all ports of a specifed datapath"
        for m in self._wait_for_sync():
            yield m
        self.apiroutine.retvalue = [p for _,p in self.managed_ports.get((vhost, datapathid), [])]
    def getallports(self, vhost = None):
        "Return all ``(datapathid, port, vhost)`` tuples, optionally filterd by vhost"
        for m in self._wait_for_sync():
            yield m
        if vhost is None:
            self.apiroutine.retvalue = [(dpid, p, vh) for (vh, dpid),v in self.managed_ports.items() for _,p in v]
        else:
            self.apiroutine.retvalue = [(dpid, p, vh) for (vh, dpid),v in self.managed_ports.items() if vh == vhost for _,p in v]
    def getportbyno(self, datapathid, portno, vhost = ''):
        "Return port with specified portno"
        portno &= 0xffff
        for m in self._wait_for_sync():
            yield m
        self.apiroutine.retvalue = self._getportbyno(datapathid, portno, vhost)
    def _getportbyno(self, datapathid, portno, vhost = ''):
        ports = self.managed_ports.get((vhost, datapathid))
        if ports is None:
            return None
        else:
            for _, p in ports:
                if p['ofport'] == portno:
                    return p
            return None
    def waitportbyno(self, datapathid, portno, timeout = 30, vhost = ''):
        "Wait for port with specified portno"
        portno &= 0xffff
        for m in self._wait_for_sync():
            yield m
        def waitinner():
            p = self._getportbyno(datapathid, portno, vhost)
            if p is not None:
                self.apiroutine.retvalue = p
            else:
                try:
                    self.wait_portnos[(vhost, datapathid, portno)] = \
                            self.wait_portnos.get((vhost, datapathid, portno),0) + 1
                    yield (OVSDBPortUpNotification.createMatcher(None, None, portno, None, vhost, datapathid),)
                except:
                    v = self.wait_portnos.get((vhost, datapathid, portno))
                    if v is not None:
                        if v <= 1:
                            del self.wait_portnos[(vhost, datapathid, portno)]
                        else:
                            self.wait_portnos[(vhost, datapathid, portno)] = v - 1
                    raise
                else:
                    self.apiroutine.retvalue = self.apiroutine.event.port
        for m in self.apiroutine.executeWithTimeout(timeout, waitinner()):
            yield m
        if self.apiroutine.timeout:
            raise OVSDBPortNotAppearException('Port ' + repr(portno) + ' does not appear before timeout')
    def getportbyname(self, datapathid, name, vhost = ''):
        "Return port with specified name"
        if isinstance(name, bytes):
            name = name.decode('utf-8')
        for m in self._wait_for_sync():
            yield m
        self.apiroutine.retvalue = self._getportbyname(datapathid, name, vhost)
    def _getportbyname(self, datapathid, name, vhost = ''):
        ports = self.managed_ports.get((vhost, datapathid))
        if ports is None:
            return None
        else:
            for _, p in ports:
                if p['name'] == name:
                    return p
            return None
    def waitportbyname(self, datapathid, name, timeout = 30, vhost = ''):
        "Wait for port with specified name"
        for m in self._wait_for_sync():
            yield m
        def waitinner():
            p = self._getportbyname(datapathid, name, vhost)
            if p is not None:
                self.apiroutine.retvalue = p
            else:
                try:
                    self.wait_names[(vhost, datapathid, name)] = \
                            self.wait_portnos.get((vhost, datapathid, name) ,0) + 1
                    yield (OVSDBPortUpNotification.createMatcher(None, name, None, None, vhost, datapathid),)
                except:
                    v = self.wait_names.get((vhost, datapathid, name))
                    if v is not None:
                        if v <= 1:
                            del self.wait_names[(vhost, datapathid, name)]
                        else:
                            self.wait_names[(vhost, datapathid, name)] = v - 1
                    raise
                else:
                    self.apiroutine.retvalue = self.apiroutine.event.port
        for m in self.apiroutine.executeWithTimeout(timeout, waitinner()):
            yield m
        if self.apiroutine.timeout:
            raise OVSDBPortNotAppearException('Port ' + repr(name) + ' does not appear before timeout')
    def getportbyid(self, id, vhost = ''):
        "Return port with the specified id. The return value is a pair: ``(datapath_id, port)``"
        for m in self._wait_for_sync():
            yield m
        self.apiroutine = self._getportbyid(id, vhost)
    def _getportbyid(self, id, vhost = ''):
        ports = self.managed_ids.get((vhost, id))
        if ports:
            return ports[0]
        else:
            return None
    def waitportbyid(self, id, timeout = 30, vhost = ''):
        "Wait for port with the specified id. The return value is a pair ``(datapath_id, port)``"
        for m in self._wait_for_sync():
            yield m
        def waitinner():
            p = self._getportbyid(id, vhost)
            if p is None:
                try:
                    self.wait_ids[(vhost, id)] = self.wait_ids.get((vhost, id), 0) + 1
                    yield (OVSDBPortUpNotification.createMatcher(None, None, None, id, vhost),)
                except:
                    v = self.wait_ids.get((vhost, id))
                    if v is not None:
                        if v <= 1:
                            del self.wait_ids[(vhost, id)]
                        else:
                            self.wait_ids[(vhost, id)] = v - 1
                    raise
                else:
                    self.apiroutine.retvalue = (self.apiroutine.event.datapathid,
                                                self.apiroutine.event.port)
            else:
                self.apiroutine.retvalue = p
        for m in self.apiroutine.executeWithTimeout(timeout, waitinner()):
            yield m
        if self.apiroutine.timeout:
            raise OVSDBPortNotAppearException('Port ' + repr(id) + ' does not appear before timeout')
    def resync(self, datapathid, vhost = ''):
        '''
        Resync with current ports
        '''
        # Sometimes when the OVSDB connection is very busy, monitor message may be dropped.
        # We must deal with this and recover from it
        # Save current manged_ports
        if (vhost, datapathid) not in self.managed_ports:
            self.apiroutine.retvalue = None
            return
        else:
            for m in callAPI(self.apiroutine, 'ovsdbmanager', 'getconnection', {'datapathid': datapathid, 'vhost':vhost}):
                yield m
            c = self.apiroutine.retvalue
            if c is not None:
                # For now, we restart the connection...
                for m in c.reconnect(False):
                    yield m
                for m in self.apiroutine.waitWithTimeout(0.1):
                    yield m
                for m in callAPI(self.apiroutine, 'ovsdbmanager', 'waitconnection', {'datapathid': datapathid,
                                                                                     'vhost': vhost}):
                    yield m
        self.apiroutine.retvalue = None



from vlcp.scripts.script import ScriptModule
from vlcp.server.module import depend, callAPI
from vlcp.service.kvdb import objectdb
from vlcp.utils.networkmodel import PhysicalNetworkMap, PhysicalNetworkSet


@depend(objectdb.ObjectDB)
class RepairPhyMapDB(ScriptModule):
    """
     before version 317b31130794650a9392eb15634a2aefaba35c28
     have problem about physicalmap don't remove weakref(logicalnetwork)
     this script repair this problem in DB
    """
    def run(self):

        saved_lgnet_keys = []
        saved_phymap_keys = []
        saved_physicalnetworkmap_to_logicalnetwork = {}

        def walk_phynetset(key,value,walk,save):
            for weak_phynet in value.set.dataset():
                try:
                    phynetobj = walk(weak_phynet.getkey())
                except KeyError:
                    pass
                else:
                    phynetmapkey = PhysicalNetworkMap.default_key(phynetobj.id)
                    try:
                        phynetmapobj = walk(phynetmapkey)
                    except KeyError:
                        pass
                    else:
                        save(phynetmapobj.getkey())
                        saved_phymap_keys.append(phynetmapobj.getkey())
                        saved_physicalnetworkmap_to_logicalnetwork[phynetmapobj.getkey()] = []
                        for lgnet in phynetmapobj.logicnetworks.dataset():
                            try:
                                lgnetobj = walk(lgnet.getkey())
                            except KeyError:
                                pass
                            else:
                                if not lgnetobj:
                                    save(lgnet.getkey())
                                    saved_lgnet_keys.append(lgnet.getkey())
                                    saved_physicalnetworkmap_to_logicalnetwork[phynetmapobj.getkey()]\
                                        .append(lgnet.getkey())

        for m in callAPI(self.apiroutine,"objectdb","walk",
                    {"keys":[PhysicalNetworkSet.default_key()],
                    "walkerdict":{PhysicalNetworkSet.default_key():walk_phynetset},
                    "requestid":1}):
            yield m

        saved_lgnet_keys = list(set(saved_lgnet_keys))
        saved_phymap_keys = list(set(saved_phymap_keys))

        print("we have find all physicalnetworkmap %s" % saved_phymap_keys)

        for k in saved_phymap_keys:
            print("# # begin to repair physicalnetwork %s" % k)
            print("# # # # remove invaild lgnet %s" % saved_physicalnetworkmap_to_logicalnetwork[k])

        def updater(keys,values):

            start = 0
            for i in range(len(saved_phymap_keys)):

                lgnet_keys = keys[1+start:start+1+len(saved_physicalnetworkmap_to_logicalnetwork[keys[start]])]
                lgnet = values[1+start:start+1+len(saved_physicalnetworkmap_to_logicalnetwork[keys[start]])]

                lgnet_dict = dict(zip(lgnet_keys,lgnet))

                for n in list(values[start].logicnetworks.dataset()):
                    if n.getkey() in lgnet_dict and lgnet_dict[n.getkey()] is None:
                        print("remove %s from %s" % (n.getkey(),keys[start]))
                        values[start].logicnetworks.dataset().discard(n)

                start = len(saved_physicalnetworkmap_to_logicalnetwork[keys[start]]) + 1

            return keys,values

        transact_keys = []

        for k in saved_phymap_keys:
            transact_keys.append(k)
            for key in saved_physicalnetworkmap_to_logicalnetwork[k]:
                transact_keys.append(key)

        if transact_keys:
            for m in callAPI(self.apiroutine,"objectdb","transact",{"keys":transact_keys,
                                                                "updater":updater}):
                yield m

        print("# # # # # # repair success !")

if __name__ == '__main__':
    RepairPhyMapDB.main()


'''
Created on 2016/3/8

:author: hubo
'''
from vlcp.config import defaultconfig
from vlcp.service.connection.tcpserver import TcpServerBase
from vlcp.protocol.redis import Redis
from vlcp.server.module import api
from vlcp.utils.redisclient import RedisClient, RedisConnectionDown
from vlcp.event.runnable import RoutineContainer
from vlcp.event.lock import Lock
from zlib import compress, decompress, error as zlib_error
import pickle
from contextlib import closing
try:
    import cPickle
except ImportError:
    pass
import json
from vlcp.utils.jsonencoder import encode_default, decode_object
import itertools
try:
    from itertools import izip
except ImportError:
    def izip(*args, **kwargs):
        return iter(zip(*args, **kwargs))

import random
from time import time

class RedisWriteConflictException(Exception):
    pass

@defaultconfig
class RedisDB(TcpServerBase):
    '''
    Create redis clients to connect to redis server
    '''
    # Default Redis connection URL
    _default_url = 'tcp://localhost/'
    # Default database selected
    _default_db = None
    # Default serialization protocol: should be "json" or "pickle"
    _default_serialize = 'json'
    # Use deflate to compress the serialized data
    _default_deflate = True
    # Pickling version: should be a number or "default"/"highest"
    _default_pickleversion = 'default'
    # Allow using cPickle
    _default_cpickle = True
    # Serialize requests on the same key in this process if a transact rolls back too many times
    _default_enterseq = 20
    # Try to use locks on the same key for all VLCP processes if a transact rolls back too many times
    _default_enterlock = 50
    # Maximum retry times for a transact
    _default_maxretry = 160
    client = True
    def __init__(self, server):
        self._redis_clients = {}
        TcpServerBase.__init__(self, server, Redis)
        self._sequencial = False
        self._sequencialsince = time()
        self._enterlock = False
        self._enterlocksince = time()
        if self.serialize == 'pickle' or self.serialize == 'cpickle' or self.serialize == 'cPickle':
            if self.serialize == 'pickle':
                if not self.cpickle:
                    p = pickle
                else:
                    p = cPickle
            else:
                p = cPickle
            if self.pickleversion is None or self.pickleversion == 'default':
                pickleversion = None
            elif self.pickleversion == 'highest':
                pickleversion = p.HIGHEST_PROTOCOL
            else:
                pickleversion = self.pickleversion
            if self.deflate:
                def _encode(obj):
                    return compress(p.dumps(obj, pickleversion), 1)
            else:
                def _encode(obj):
                    return p.dumps(obj, pickleversion)
            self._encode = _encode
            if self.deflate:
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        try:
                            return p.loads(decompress(data))
                        except zlib_error:
                            return p.loads(data)
            else:
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        return p.loads(data)
            self._decode = _decode
        else:
            if self.deflate:
                def _encode(obj):
                    return compress(json.dumps(obj, default=encode_default).encode('utf-8'), 1)
                self._encode = _encode
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    else:
                        try:
                            data = decompress(data)
                        except zlib_error:
                            pass
                        if not isinstance(data, str) and isinstance(data, bytes):
                            data = data.decode('utf-8')
                        return json.loads(data, object_hook=decode_object)
                self._decode = _decode
            else:
                def _encode(obj):
                    return json.dumps(obj, default=encode_default).encode('utf-8')
                self._encode = _encode
                def _decode(data):
                    if data is None:
                        return None
                    elif isinstance(data, Exception):
                        raise data
                    elif not isinstance(data, str) and isinstance(data, bytes):
                        data = data.decode('utf-8')
                    return json.loads(data, object_hook=decode_object)
                self._decode = _decode                
        self.appendAPI(api(self.getclient),
                       api(self.get, self.apiroutine),
                       api(self.set, self.apiroutine),
                       api(self.delete, self.apiroutine),
                       api(self.mget, self.apiroutine),
                       api(self.mset, self.apiroutine),
                       api(self.update, self.apiroutine),
                       api(self.mupdate, self.apiroutine),
                       api(self.updateall, self.apiroutine),
                       api(self.updateallwithtime, self.apiroutine),
                       api(self.listallkeys, self.apiroutine))
    def _client_class(self, config, protocol, vhost):
        db = getattr(config, 'db', None)
        def _create_client(url, protocol, scheduler = None, key = None, certificate = None, ca_certs = None, bindaddress = None):
            c = RedisClient(url, db, protocol)
            if key:
                c.key = key
            if certificate:
                c.certificate = certificate
            if ca_certs:
                c.ca_certs = ca_certs
            self._redis_clients[vhost] = c
            return c.make_connobj(self.apiroutine)
        return _create_client
    def getclient(self, vhost = ''):
        "Return a tuple of ``(redisclient, encoder, decoder)`` for specified vhost"
        return (self._redis_clients.get(vhost), self._encode, self._decode)
    def get(self, key, timeout = None, vhost = ''):
        "Get value from key"
        c = self._redis_clients.get(vhost)
        if c is None:
            raise ValueError('vhost ' + repr(vhost) + ' is not defined')
        if timeout is not None:
            if timeout <= 0:
                for m in c.batch_execute(self.apiroutine, ('MULTI',), 
                                                        ('GET', key),
                                                        ('DEL', key),
                                                        ('EXEC',)):
                    yield m
            else:
                for m in c.batch_execute(self.apiroutine, ('MULTI',),
                                                        ('GET', key),
                                                        ('PEXPIRE', key, int(timeout * 1000)),
                                                        ('EXEC',)):
                    yield m
            r = self.apiroutine.retvalue[3][0]
            if isinstance(r, Exception):
                raise r
            self.apiroutine.retvalue = self._decode(r)
        else:
            for m in c.execute_command(self.apiroutine, 'GET', key):
                yield m
            self.apiroutine.retvalue = self._decode(self.apiroutine.retvalue)
    def set(self, key, value, timeout = None, vhost = ''):
        "Set value to key, with an optional timeout"
        c = self._redis_clients.get(vhost)
        if timeout is None:
            for m in c.execute_command(self.apiroutine, 'SET', key, self._encode(value)):
                yield m
        elif timeout <= 0:
            for m in c.execute_command(self.apiroutine, 'DEL', key):
                yield m
        else:
            for m in c.execute_command(self.apiroutine, 'PSETEX', key, int(timeout * 1000), self._encode(value)):
                yield m
        self.apiroutine.retvalue = None
    def delete(self, key, vhost = ''):
        "Delete a key from the storage"
        c = self._redis_clients.get(vhost)
        for m in c.execute_command(self.apiroutine, 'DEL', key):
            yield m
        self.apiroutine.retvalue = None
    def mget(self, keys, vhost = ''):
        "Get multiple values from multiple keys"
        if not keys:
            self.apiroutine.retvalue = []
            return
        c = self._redis_clients.get(vhost)
        for m in c.execute_command(self.apiroutine, 'MGET', *keys):
            yield m
        self.apiroutine.retvalue = [self._decode(r) for r in self.apiroutine.retvalue]
    def mset(self, kvpairs, timeout = None, vhost = ''):
        "Set multiple values on multiple keys"
        if not kvpairs:
            return
        c = self._redis_clients.get(vhost)
        d = kvpairs
        if hasattr(d, 'items'):
            d = d.items()
        if timeout is not None and timeout <= 0:
            for m in c.execute_command(self.apiroutine, 'DEL', *[k for k,_ in d]):
                yield m
        else:
            if timeout is None:
                for m in c.execute_command(self.apiroutine, 'MSET',
                                           *list(
                                               itertools.chain.from_iterable(
                                                   (k, self._encode(v)) for (k,v) in d
                                                )
                                            )
                                           ):
                    yield m
            else:
                # Use a transact
                ptimeout = int(timeout * 1000)
                
                for m in c.batch_execute(self.apiroutine,
                                         *(
                                             (
                                                 ('MULTI',),
                                                 ('MSET',) + \
                                                    tuple(itertools.chain.from_iterable(
                                                            (k, self._encode(v)) for (k,v) in d
                                                          )
                                                        )
                                              ) + \
                                              tuple(
                                                  ('PEXPIRE', k, ptimeout) for k, _ in d
                                              ) + \
                                              (
                                                  ('EXEC',),
                                              )
                                           )
                                        ):
                    yield m
        self.apiroutine.retvalue = None
    def _retry_write(self, process, vhost):
        c = self._redis_clients.get(vhost)
        # Always try once first
        while True:
            for m in c.get_connection(self.apiroutine):
                yield m
            newconn = self.apiroutine.retvalue
            with newconn.context(self.apiroutine):
                try:
                    for m in process(newconn):
                        yield m
                except RedisConnectionDown:
                    continue
                except RedisWriteConflictException:
                    break
                else:
                    return
        enterseq = self.enterseq
        enterlock = self.enterlock
        for i in range(0, self.maxretry):
            for m in c.get_connection(self.apiroutine):
                yield m
            newconn = self.apiroutine.retvalue
            with newconn.context(self.apiroutine):
                if self._sequencial:
                    curr_time = time()
                    if self._sequencialsince + 1.0 < curr_time:
                        self._sequencial = False
                    else:
                        self._sequencialsince = curr_time
                if self._sequencial:
                    # First make all the conflicted updates in this process retry in sequence
                    l = Lock(self, self.scheduler)
                    for m in l.lock(self.apiroutine):
                        yield m
                    with l:
                        try:
                            for m in process(newconn):
                                yield m
                        except RedisConnectionDown:
                            continue
                        except RedisWriteConflictException:
                            if i > enterseq and not self._sequencial:
                                self._sequencial = True
                                self._sequencialsince = time()
                            if i > enterlock:
                                try:
                                    # Maybe too many VLCP processes are trying to write the same Redis server
                                    # Set a special flag in Redis with very short expire time
                                    # If multiple VLCP processes tries to set the flag at the same time, some
                                    # will be blocked for a short time to leave time for the winner
                                    for i in range(0, 100):
                                        for m in newconn.execute_command(self.apiroutine,
                                                                         'SET', 'vlcp._reserved.redisdb.connwriteblock',
                                                                         '1', 'PX', '100', 'NX'):
                                            yield m
                                        if self.apiroutine.retvalue is not None:
                                            break
                                        else:
                                            for m in self.apiroutine.waitWithTimeout(0.02):
                                                yield m
                                except Exception:
                                    self._logger.warning('Exception raised on waiting for a lock, will ignore and continue', exc_info = True)
                                    for m in self.apiroutine.waitWithTimeout(0.1):
                                        yield m
                        else:
                            return
                else:
                    try:
                        for m in process(newconn):
                            yield m
                    except RedisConnectionDown:
                        continue
                    except RedisWriteConflictException:
                        if i > enterseq and not self._sequencial:
                            self._sequencial = True
                            self._sequencialsince = time()
                    else:
                        return
        raise RedisWriteConflictException('Transaction still fails after many retries')
    def update(self, key, updater, timeout = None, vhost = ''):
        '''
        Update in-place with a custom function
        
        :param key: key to update
        
        :param updater: ``func(k,v)``, should return a new value to update, or return None to delete. The function
                        may be call more than once.
        
        :param timeout: new timeout
        
        :returns: the updated value, or None if deleted
        '''
        def _process(newconn):
            for m in newconn.execute_command(self.apiroutine, 'WATCH', key):
                yield m
            for m in newconn.execute_command(self.apiroutine, 'GET', key):
                yield m
            v = self._decode(self.apiroutine.retvalue)
            try:
                v = updater(key, v)
                if v is not None:
                    ve = self._encode(v)
            except Exception as exc:
                for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                    yield m
                raise exc
            if timeout is not None and timeout <= 0 or v is None:
                set_command = ('DEL', key)
            elif timeout is None:
                set_command = ('SET', key, ve)
            else:
                set_command = ('PSETEX', key, int(timeout * 1000), ve)
            for m in newconn.batch_execute(self.apiroutine, ('MULTI',),
                                                            set_command,
                                                            ('EXEC',)):
                yield m
            r = self.apiroutine.retvalue[2]
            if r is not None:
                # Succeeded
                self.apiroutine.retvalue = v
                return
            else:
                raise RedisWriteConflictException('Transaction still fails after many retries: key=' + repr(key))
        with closing(self._retry_write(_process, vhost)) as g:
            for m in g:
                yield m
    def mupdate(self, keys, updater, timeout = None, vhost = ''):
        "Update multiple keys in-place with a custom function, see update. Either all success, or all fail."
        if not keys:
            self.apiroutine.retvalue = []
            return
        def _process(newconn):
            for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                yield m
            for m in newconn.execute_command(self.apiroutine, 'MGET', *keys):
                yield m
            values = self.apiroutine.retvalue
            try:
                values = [updater(k, self._decode(v)) for k,v in izip(keys,values)]
                keys_deleted = [k for k,v in izip(keys,values) if v is None]
                values_encoded = [(k,self._encode(v)) for k,v in izip(keys,values) if v is not None]
            except Exception as exc:
                for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                    yield m
                raise exc
            if timeout is None:
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            elif timeout <= 0:
                set_commands = (('DEL',) + tuple(keys),)
            else:
                ptimeout = int(timeout * 1000)
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            if not set_commands:
                for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                    yield m
                self.apiroutine.retvalue = values
                return
            for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                            set_commands + \
                                                            (('EXEC',),))):
                yield m
            r = self.apiroutine.retvalue[-1]
            if r is not None:
                # Succeeded
                self.apiroutine.retvalue = values
                return
            else:
                raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
        with closing(self._retry_write(_process, vhost)) as g:
            for m in g:
                yield m
    def updateall(self, keys, updater, timeout = None, vhost = ''):
        """
        Update multiple keys in-place, with a function ``updater(keys, values)``
        which returns ``(updated_keys, updated_values)``.
        
        Either all success or all fail
        """
        def _process(newconn):
            if keys:
                for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                    yield m
                for m in newconn.execute_command(self.apiroutine, 'MGET', *keys):
                    yield m
                values = self.apiroutine.retvalue
            else:
                values = []
            try:
                new_keys, new_values = updater(keys, [self._decode(v) for v in values])
                keys_deleted = [k for k,v in izip(new_keys, new_values) if v is None]
                values_encoded = [(k,self._encode(v)) for k,v in izip(new_keys, new_values) if v is not None]
            except Exception as exc:
                if keys:
                    for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                        yield m
                raise exc
            if timeout is None:
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            elif timeout <= 0:
                if new_keys:
                    set_commands = (('DEL',) + tuple(new_keys),)
                else:
                    set_commands = ()
            else:
                ptimeout = int(timeout * 1000)
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            if not set_commands:
                for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                    yield m
                self.apiroutine.retvalue = values
                return
            for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                            set_commands + \
                                                            (('EXEC',),))):
                yield m
            r = self.apiroutine.retvalue[-1]
            if r is not None:
                # Succeeded
                self.apiroutine.retvalue = (new_keys, new_values)
                return
            else:
                raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
        with closing(self._retry_write(_process, vhost)) as g:
            for m in g:
                yield m
    def updateallwithtime(self, keys, updater, timeout = None, vhost = ''):
        """
        Update multiple keys in-place, with a function ``updater(keys, values, timestamp)``
        which returns ``(updated_keys, updated_values)``.
        
        Either all success or all fail.
        
        Timestamp is a integer standing for current time in microseconds.
        """
        def _process(newconn):
            if keys:
                for m in newconn.execute_command(self.apiroutine, 'WATCH', *keys):
                    yield m
                for m in newconn.batch_execute(self.apiroutine, ('MGET',) + tuple(keys),
                                                                ('TIME',)):
                    yield m
                values, time_tuple = self.apiroutine.retvalue
            else:
                for m in newconn.execute_command(self.apiroutine, 'TIME'):
                    yield m
                values = []
                time_tuple = self.apiroutine.retvalue
            server_time = int(time_tuple[0]) * 1000000 + int(time_tuple[1])
            try:
                new_keys, new_values = updater(keys, [self._decode(v) for v in values], server_time)
                keys_deleted = [k for k,v in izip(new_keys, new_values) if v is None]
                values_encoded = [(k,self._encode(v)) for k,v in izip(new_keys, new_values) if v is not None]
            except Exception as exc:
                if keys:
                    for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                        yield m
                raise exc
            if timeout is None:
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            elif timeout <= 0:
                if new_keys:
                    set_commands = (('DEL',) + tuple(new_keys),)
                else:
                    set_commands = ()
            else:
                ptimeout = int(timeout * 1000)
                set_commands_list = []
                if values_encoded:
                    set_commands_list.append(('MSET',) + tuple(itertools.chain.from_iterable(values_encoded)))
                    set_commands_list.extend(('PEXPIRE', k, ptimeout) for k,_ in values_encoded)
                if keys_deleted:
                    set_commands_list.append(('DEL',) + tuple(keys_deleted))
                set_commands = tuple(set_commands_list)
            if not set_commands:
                for m in newconn.execute_command(self.apiroutine, 'UNWATCH'):
                    yield m
                self.apiroutine.retvalue = values
                return
            for m in newconn.batch_execute(self.apiroutine, *((('MULTI',),) + \
                                                            set_commands + \
                                                            (('EXEC',),))):
                yield m
            r = self.apiroutine.retvalue[-1]
            if r is not None:
                # Succeeded
                self.apiroutine.retvalue = (new_keys, new_values)
                return
            else:
                raise RedisWriteConflictException('Transaction still fails after many retries: keys=' + repr(keys))
        with closing(self._retry_write(_process, vhost)) as g:
            for m in g:
                yield m
    def listallkeys(self, vhost = ''):
        '''
        Return all keys in the KVDB. For management purpose.
        '''
        c = self._redis_clients.get(vhost)
        if c is None:
            raise ValueError('vhost ' + repr(vhost) + ' is not defined')
        for m in c.execute_command(self.apiroutine, 'KEYS', '*'):
            yield m




from .defs import common, openflow10, openflow13, definations
from .openflow import Openflow, OpenflowAsyncMessageEvent, OpenflowConnectionStateEvent, OpenflowExperimenterMessageEvent, OpenflowPresetupMessageEvent,\
    OpenflowResponseEvent



'''
Created on 2016/5/27

:author: hubo
'''

from vlcp.server.module import Module, ModuleLoadStateChanged
from vlcp.config.config import defaultconfig
import pkgutil

@defaultconfig
class AutoLoad(Module):
    '''
    Auto load some modules from a package. Usually used to load network plugins.
    '''
    autosuccess = False
    # Auto load packages from some packages
    _default_autoloadpackages = ('vlcp.service.sdn.plugins',)
    def __init__(self, server):
        Module.__init__(self, server)
    def load(self, container):
        for m in Module.load(self, container):
            yield m
        loadmodules = []
        for p in self.autoloadpackages:
            try:
                def onerror(name):
                    self._logger.warning("Autoload package %r on package %r failed", name, p)
                pkg = __import__(p, fromlist = ('dummy',))
                for _, name, ispkg in pkgutil.walk_packages(pkg.__path__, p + '.', onerror):
                    if not ispkg:
                        try:
                            pymod = __import__(name, fromlist = ('dummy',))
                            for m in vars(pymod).values():
                                if isinstance(m, type) and issubclass(m, Module) and getattr(m, '__module__', '') == name:
                                    loadmodules.append(m)
                        except Exception:
                            self._logger.warning('Autoload module %r failed', name, exc_info = True)
            except Exception:
                self._logger.warning('Autoload package %r failed', p, exc_info = True)
        if loadmodules:
            for m in container.executeAll([self.server.moduleloader.loadmodule(m) for m in loadmodules],
                                          self.server.moduleloader, ()):
                yield m
        for m in self.changestate(ModuleLoadStateChanged.SUCCEEDED, container):
            yield m
    def unload(self, container, force=False):
        for m in Module.unload(self, container, force=force):
            yield m
    


'''
Created on 2016/10/18

:author: hubo
'''


class IndexedHeap(object):
    '''
    A heap with indices
    '''
    def __init__(self):
        self.heap = []
        self.index = {}
    def push(self, value, priority):
        if value in self.index:
            self.setpriority(value, priority)
        else:
            self.heap.append((priority, value))
            self.index[value] = len(self.heap) - 1
            self._siftup(len(self.heap) - 1)
    def remove(self, value):
        if value in self.index:
            pos = self.index[value]
            del self.index[value]
            if pos == len(self.heap) - 1:
                del self.heap[-1]
                return
            ci = self.heap[pos]
            li = self.heap.pop()
            self.heap[pos] = li
            self.index[li[1]] = pos
            if li[0] > ci[0]:
                self._siftdown(pos)
            else:
                self._siftup(pos)
    def pop(self):
        if not self.heap:
            raise IndexError('pop from an empty heap')
        ret = self.heap[0]
        del self.index[ret[1]]
        li = self.heap.pop()
        if not self.heap:
            return ret[1]
        self.heap[0] = li
        self.index[li[1]] = 0
        self._siftdown(0)
        return ret[1]
    def poppush(self, value, priority):
        if not self.heap:
            raise IndexError('pop from an empty heap')
        ret = self.heap[0]
        del self.index[ret[1]]
        self.heap[0] = (priority, value)
        self.index[value] = 0
        self._siftdown(0)
        return ret[1]
    def pushpop(self, value, priority):
        if not self.heap or priority < self.heap[0][0]:
            return value
        return self.poppush(value, priority)
    def setpriority(self, value, priority):
        pos = self.index[value]
        temp = self.heap[pos]
        self.heap[pos] = (priority, value)
        if temp[0] < priority:
            self._siftdown(pos)
        else:
            self._siftup(pos)
    def replace(self, value, value2):
        pos = self.index[value]
        del self.index[value]
        self.heap[pos] = (self.heap[pos][0], value2)
        self.index[value2] = pos
    def clear(self):
        self.index.clear()
        del self.heap[:]
    def top(self):
        return self.heap[0][1]
    def topPriority(self):
        return self.heap[0][0]
    def _siftup(self, pos):
        temp = self.heap[pos]
        while pos > 0:
            pindex = (pos - 1) // 2
            pt = self.heap[pindex]
            if pt[0] > temp[0]:
                self.heap[pos] = pt
                self.index[pt[1]] = pos
            else:
                break
            pos = pindex
        self.heap[pos] = temp
        self.index[temp[1]] = pos
    def _siftdown(self, pos):
        temp = self.heap[pos]
        l = len(self.heap)
        while pos * 2 + 1 < l:
            cindex = pos * 2 + 1
            pt = self.heap[cindex]
            if cindex + 1 < l and self.heap[cindex+1][0] < pt[0]:
                cindex = cindex + 1
                pt = self.heap[cindex]
            if pt[0] < temp[0]:
                self.heap[pos] = pt
                self.index[pt[1]] = pos
            else:
                break
            pos = cindex
        self.heap[pos] = temp
        self.index[temp[1]] = pos
    def __len__(self):
        return len(self.heap)
    def __nonzero__(self):
        return bool(self.heap)
    def __contains__(self, value):
        return value in self.index



'''
Created on 2016/9/28

:author: hubo

Future is a helper class to simplify the process of retrieving a result from other routines.
The implementation is straight-forward: first check the return value, if not set, wait for
the event. Multiple routines can wait for the same Future object.

The interface is similar to asyncio, but:

- Cancel is not supported - you should terminate the sender routine instead. But RoutineFuture supports
  close() (and cancel() which is the same)

- Callback is not supported - start a subroutine to wait for the result instead.

- result() returns None if the result is not ready; exception() is not supported.

- New wait() generator function: get the result, or wait for the result until available. It is
  always the recommended way to use a future; result() is not recommended.

- ensure_result() which returns a context manager: this should be used in the sender routine,
  to ensure that a result is always set after exit the with scope. If the result is not set,
  it is set to None; if an exception is raised, it is set with set_exception.

'''
from vlcp.event.event import withIndices, Event
from contextlib import contextmanager

@withIndices('futureobj')
class FutureEvent(Event):
    pass


class Future(object):
    """
    Basic future object
    """
    def __init__(self, scheduler):
        self._scheduler = scheduler
    def done(self):
        '''
        :return: True if the result is available; False otherwise.
        '''
        return hasattr(self, '_result')
    def result(self):
        '''
        :return: None if the result is not ready, the result from set_result, or raise the exception
                 from set_exception. If the result can be None, it is not possible to tell if the result is
                 available; use done() to determine that.
        '''
        try:
            r = getattr(self, '_result')
        except AttributeError:
            return None
        else:
            if hasattr(self, '_exception'):
                raise self._exception
            else:
                return r
    def wait(self, container):
        '''
        :param container: container of current routine
        
        :return: The result, or raise the exception from set_exception. The result is returned to container.retvalue.
        '''
        if hasattr(self, '_result'):
            if hasattr(self, '_exception'):
                raise self._exception
            else:
                container.retvalue = self._result
        else:
            yield (FutureEvent.createMatcher(self),)
            if hasattr(container.event, 'exception'):
                raise container.event.exception
            else:
                container.retvalue = container.event.result
    def set_result(self, result):
        '''
        Set the result to Future object, wake up all the waiters
        
        :param result: result to set
        '''
        if hasattr(self, '_result'):
            raise ValueError('Cannot set the result twice')
        self._result = result
        self._scheduler.emergesend(FutureEvent(self, result = result))
    def set_exception(self, exception):
        '''
        Set an exception to Future object, wake up all the waiters
        
        :param exception: exception to set
        '''
        if hasattr(self, '_result'):
            raise ValueError('Cannot set the result twice')
        self._result = None
        self._exception = exception
        self._scheduler.emergesend(FutureEvent(self, exception = exception))
    @contextmanager
    def ensure_result(self, supress_exception = False, defaultresult = None):
        '''
        Context manager to ensure returning the result
        '''
        try:
            yield self
        except Exception as exc:
            self.set_exception(exc)
            if not supress_exception:
                raise
        else:
            if not self.done():
                self.set_result(defaultresult)


class FutureCancelledException(Exception):
    pass


class RoutineFuture(Future):
    '''
    Quick wrapper to create a subroutine and return the result to a Future object
    '''
    def __init__(self, subprocess, container):
        '''
        Start the subprocess
        
        :param subprocess: a generator process, which returns the result to container.retvalue on exit
        
        :param container: the routine container to run the subprocess with
        '''
        Future.__init__(self, container.scheduler)
        def _subroutine():
            with self.ensure_result(True):
                try:
                    for m in subprocess:
                        yield m
                except GeneratorExit:
                    raise FutureCancelledException('close is called before result returns')
                else:
                    self.set_result(container.retvalue)
        self._routine = container.subroutine(_subroutine())
    def close(self):
        '''
        Terminate the subprocess
        '''
        if not self.done():
            self._routine.close()
    def cancel(self):
        '''
        Same as close()
        '''
        self.close()



'''
Created on 2015/7/8

:author: hubo
'''
from vlcp.protocol import Protocol
from vlcp.config import defaultconfig
from .defs import common, openflow10
from .defs.definations import definations
from vlcp.event.core import Event, withIndices, QuitException
from vlcp.event.runnable import RoutineContainer, RoutineException
from vlcp.event.connection import ConnectionWriteEvent, ConnectionControlEvent
import logging
import os
from namedstruct import dump


@withIndices('datapathid', 'auxiliaryid', 'state', 'connection', 'connmark', 'createby')
class OpenflowConnectionStateEvent(Event):
    """
    Event when connection state changes
    """
    # Connection Setup
    CONNECTION_SETUP = 'setup'
    # Connection Down
    CONNECTION_DOWN = 'down'

@withIndices('datapathid', 'auxiliaryid', 'connection', 'connmark', 'xid', 'iserror', 'createby')
class OpenflowResponseEvent(Event):
    """
    Event for an OpenFlow response is received
    """
    pass

@withIndices('type', 'datapathid', 'auxiliaryid', 'tableid', 'cookie', 'connection', 'connmark', 'createby')
class OpenflowAsyncMessageEvent(Event):
    """
    Event for an async message is received
    """
    pass

@withIndices('connection', 'type', 'connmark', 'createby')
class OpenflowPresetupMessageEvent(Event):
    """
    Event for messages before connection setup
    """
    pass

@withIndices('experimenter', 'exptype', 'datapathid', 'auxiliaryid', 'connection', 'connmark', 'createby')
class OpenflowExperimenterMessageEvent(Event):
    """
    Event for experimenter messages
    """
    pass

class OpenflowProtocolException(Exception):
    """
    Critical protocol break exception
    """
    pass

class OpenflowErrorResultException(Exception):
    """
    OpenFlow returns error
    """
    pass

@defaultconfig
class Openflow(Protocol):
    '''
    Openflow control protocol
    '''
    _default_persist = True
    # Default OpenFlow port
    _default_defaultport = common.OFP_TCP_PORT
    # Allowed versions for OpenFlow handshake; should be one or both of OFP10_VERSION and OFP13_VERSION
    _default_allowedversions = (common.OFP10_VERSION, common.OFP13_VERSION)
    # Disconnect when OFPT_HELLO message is not received for a long time
    _default_hellotimeout = 10
    # Disconnect when OFPT_FEATURES_REQUEST message does not get response
    _default_featurerequesttimeout = 30
    # Send OFPT_ECHO packet when connection is idle
    _default_keepalivetime = 10
    # When OFPT_ECHO packet does not get response in the specified time, disconnect
    _default_keepalivetimeout = 3
    _default_createqueue = True
    _default_buffersize = 65536
    # Show debugging messages in log
    _default_debugging = False
    # Disable nicira extension; when you are not using VLCP with OpenvSwitch (e.g. using with
    # physical switches) you should set this to True 
    _default_disablenxext = False
    # Disable using ofp_action_group in another group: this is only supported in OpenvSwitch 2.5+,
    # so it is disabled by default
    _default_disablechaining = True
    _logger = logging.getLogger(__name__ + '.Openflow')
    _default_tcp_nodelay = True
    def __init__(self, allowedVersions = None):
        '''
        Constructor
        
        :param allowedVersions: if specified, should be a tuple of allowed OpenFlow versions.
        '''
        Protocol.__init__(self)
        if allowedVersions is not None:
            self.allowedversions = allowedVersions
    def handler(self, connection):
        try:
            # Create a hello message
            connmark = connection.connmark
            hello = common.ofp_hello.new()
            hello.header.version = max(self.allowedversions)
            versionbitmap = common.ofp_hello_elem_versionbitmap.new()
            versionStart = 0
            thisBitmap = 0
            for v in sorted(self.allowedversions):
                while v > versionStart + 32:
                    versionbitmap.bitmaps.append(thisBitmap)
                    thisBitmap = 0
                    versionStart += 32
                thisBitmap = thisBitmap | (1<<(v - versionStart))
            versionbitmap.bitmaps.append(thisBitmap)
            hello.elements.append(versionbitmap)
            write = self.formatrequest(hello, connection)
            for m in connection.write(write, False):
                yield m
            # Wait for a hello
            hellomatcher = OpenflowPresetupMessageEvent.createMatcher(connection = connection)
            for m in connection.waitWithTimeout(self.hellotimeout, hellomatcher):
                yield m
            if connection.timeout:
                # Drop the connection
                raise OpenflowProtocolException('Did not receive hello message before timeout')
            else:
                msg = connection.event.message
                if msg.header.type != common.OFPT_HELLO:
                    raise OpenflowProtocolException('The first packet on this connection is not OFPT_HELLO')
                else:
                    helloversion = None
                    usebitmap = False
                    for e in msg.elements:
                        if e.type == common.OFPHET_VERSIONBITMAP:
                            # There is a bitmap
                            for v in reversed(sorted(self.allowedversions)):
                                bitmapIndex = v // 32
                                bitmapPos = (v & 31)
                                if len(e.bitmaps) < bitmapIndex:
                                    continue
                                if e.bitmaps[bitmapIndex] & (1 << bitmapPos):
                                    helloversion = v
                                    break
                            usebitmap = True
                            break
                    if not usebitmap:
                        helloversion = min(max(self.allowedversions), msg.header.version)
                    if helloversion is None or helloversion not in self.allowedversions:
                        self._logger.warning('Remote switch openflow protocol version is not compatible. Their hello message: %r, we expect version: %r. Connection = %r', common.dump(msg), self.allowedversions, connection)
                        # Hello fail
                        hellofail = common.ofp_error_msg.new()
                        hellofail.header.version = max(self.allowedversions)
                        hellofail.type = common.OFPET_HELLO_FAILED
                        hellofail.code = common.OFPHFC_INCOMPATIBLE
                        if helloversion is None:
                            hellofail.data = b'A common version is not found from the bitmap\x00'
                        else:
                            hellofail.data = ('Openflow version is not supported\x00' % (common.ofp_version.getName(helloversion, str(helloversion)),)).encode()
                        write = self.formatreply(hellofail, msg, connection)
                        for m in connection.write(write):
                            yield m
                        for m in connection.reset(False, connmark):
                            yield m
                        raise GeneratorExit
                    else:
                        # Still we may receive a hello fail from the other side, we should expect that.
                        # The error message may come before feature request is sent.
                        err_matcher = OpenflowPresetupMessageEvent.createMatcher(connection = connection, type = common.OFPT_ERROR)
                        # Send a feature request message
                        connection.openflowversion = helloversion
                        currdef = definations[helloversion]
                        connection.openflowdef = currdef
                        # Feature request message has no body
                        featurereq = currdef.ofp_msg.new()
                        featurereq.header.type = currdef.OFPT_FEATURES_REQUEST
                        write = self.formatrequest(featurereq, connection)
                        try:
                            for m in connection.withException(connection.write(write, False), err_matcher):
                                yield m
                            featurereply_matcher = OpenflowPresetupMessageEvent.createMatcher(connection = connection, type = currdef.OFPT_FEATURES_REPLY)
                            for m in connection.waitWithTimeout(self.featurerequesttimeout, featurereply_matcher, err_matcher):
                                yield m
                            if connection.timeout:
                                raise OpenflowProtocolException('Remote switch did not response to feature request.')
                            elif connection.matcher is err_matcher:
                                self._logger.warning('Error while request feature: %r Connection = %r', connection.event.message, connection)
                                raise OpenflowProtocolException('Error while request feature: %r' % (connection.event.message,))
                            else:
                                msg = connection.event.message
                                connection.openflow_featuresreply = msg
                                connection.openflow_datapathid = msg.datapath_id
                                connection.openflow_auxiliaryid = getattr(msg, 'auxiliary_id', 0)
                                connection.openflow_capabilities = msg.capabilities
                                connection.openflow_n_buffers = msg.n_buffers
                                connection.openflow_n_tables = msg.n_tables
                                statechange = OpenflowConnectionStateEvent(connection.openflow_datapathid, connection.openflow_auxiliaryid, OpenflowConnectionStateEvent.CONNECTION_SETUP, connection, connection.connmark, self)
                                for m in connection.waitForSend(statechange):
                                    yield m
                                for msg in connection.openflow_msgbuffer:
                                    e = self._createevent(connection, msg)
                                    if e is not None:
                                        for m in connection.waitForSend(e):
                                            yield m
                        except RoutineException as exc:
                            self._logger.warning('Remote report hello fail: %r Connection = %r', common.dump(exc.arguments[1].message), connection)
                            for m in connection.reset(True, connmark):
                                yield m
                            raise GeneratorExit
        except QuitException:
            pass
        except GeneratorExit:
            pass
        except:
            self._logger.exception('Unexpected exception on processing openflow protocols, Connection = %r', connection)
            for m in connection.reset(True, connmark):
                yield m
    def formatrequest(self, request, connection, assignxid = True):
        if assignxid:
            self.assignxid(request, connection)
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = request._tobytes())
        if self.debugging:
            self._logger.debug('message formatted: %r', common.dump(request))
        return c
    def assignxid(self, request, connection):
        request.header.xid = connection.xid
        connection.xid += 1
        if connection.xid > 0xffffffffffffffff:
            # Skip xid = 0 for special response
            connection.xid = 1        
    def formatreply(self, reply, request, connection):
        reply.header.xid = request.header.xid
        c = ConnectionWriteEvent(connection = connection, connmark = connection.connmark, data = reply._tobytes())
        if self.debugging:
            self._logger.debug('message formatted: %r', common.dump(request))
        return c
    def replymatcher(self, request, connection, iserror = None):
        """
        Create an event matcher to match a reply to this request
        """
        matcherparam = {'datapathid': connection.openflow_datapathid, 'auxiliaryid': connection.openflow_auxiliaryid, 'connection' : connection, 'connmark': connection.connmark, 
                        'xid': request.header.xid}
        if iserror is not None:
            matcherparam['iserror'] = iserror
        return OpenflowResponseEvent.createMatcher(**matcherparam)
    def statematcher(self, connection, state = OpenflowConnectionStateEvent.CONNECTION_DOWN, currentconn = True):
        """
        Create an event matcher to match the connection state of this connection
        """
        if currentconn:
            return OpenflowConnectionStateEvent.createMatcher(connection.openflow_datapathid, connection.openflow_auxiliaryid, state, connection, connection.connmark)
        else:
            return OpenflowConnectionStateEvent.createMatcher(connection.openflow_datapathid, connection.openflow_auxiliaryid, state)
    def querywithreply(self, request, connection, container, raiseonerror = True):
        """
        Send an OpenFlow normal request, wait for the response of this request. The request must have
        exactly one response. The reply is stored to `container.openflow_reply`
        """
        for m in connection.write(self.formatrequest(request, connection), False):
            yield m
        reply = self.replymatcher(request, connection)
        conndown = self.statematcher(connection)
        yield (reply, conndown)
        if container.matcher is conndown:
            raise OpenflowProtocolException('Connection is down before reply received')
        container.openflow_reply = container.event.message
        if container.event.iserror:
            raise OpenflowErrorResultException('An error message is returned: ' + repr(dump(container.event.message)))
    def querymultipart(self, request, connection, container, raiseonerror = True):
        """
        Send a multipart request, wait for all the responses. Return a list of reply messages
        by `container.openflow_reply`
        """
        for m in connection.write(self.formatrequest(request, connection), False):
            yield m
        reply = self.replymatcher(request, connection)
        conndown = self.statematcher(connection)
        messages = []
        while True:
            yield (reply, conndown)
            if container.matcher is conndown:
                raise OpenflowProtocolException('Connection is down before reply received')
            msg = container.event.message
            messages.append(msg)
            if msg.header.type == common.OFPT_ERROR or not (msg.flags & common.OFPSF_REPLY_MORE):
                if msg.header.type == common.OFPT_ERROR and raiseonerror:
                    container.openflow_reply = messages
                    raise OpenflowErrorResultException('An error message is returned: ' + repr(dump(msg)))
                break
        container.openflow_reply = messages
    def batch(self, requests, connection, container, raiseonerror = True):
        """
        Send multiple requests, return when all the requests are done. Requests can have no responses.
        `container.openflow_reply` is the list of messages in receiving order. `container.openflow_replydict`
        is the dictionary `{request:reply}`. The attributes are set even if an OpenflowErrorResultException is raised.
        """
        for r in requests:
            self.assignxid(r, connection)
        replymatchers = dict((self.replymatcher(r, connection), r) for r in requests)
        replydict = {}
        replymessages = []
        conndown = self.statematcher(connection)
        firsterror = [None]
        def callback(event, matcher):
            if matcher is conndown:
                raise OpenflowProtocolException('Connection is down before reply received')
            msg = event.message
            if event.iserror and not firsterror[0]:
                firsterror[0] = msg
            replydict.setdefault(matcher, []).append(msg)
            replymessages.append(msg)
        def batchprocess():
            for r in requests:
                for m in connection.write(self.formatrequest(r, connection, False), False):
                    yield m
            barrier = connection.openflowdef.ofp_msg.new()
            barrier.header.type = connection.openflowdef.OFPT_BARRIER_REQUEST
            for m in container.waitForSend(self.formatrequest(barrier, connection)):
                yield m
            barrierreply = self.replymatcher(barrier, connection)
            yield (barrierreply,)
        for m in container.withCallback(batchprocess(), callback, conndown, *replymatchers.keys()):
            yield m
        container.openflow_reply = replymessages
        container.openflow_replydict = dict((replymatchers[k],v) for k,v in replydict.items())
        if firsterror[0] and raiseonerror:
            raise OpenflowErrorResultException('One or more error message is returned from a batch process, the first is: '
                                               + repr(dump(firsterror[0])))
    def init(self, connection):
        for m in Protocol.init(self, connection):
            yield m
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, OpenflowPresetupMessageEvent.createMatcher(connection = connection), ('presetup', connection)))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, OpenflowConnectionStateEvent.createMatcher(connection = connection), ('connstate', connection)))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority + 1, OpenflowResponseEvent.createMatcher(connection = connection), ('response', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, OpenflowAsyncMessageEvent.createMatcher(connection = connection), ('async', connection), self.messagequeuesize))
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.messagepriority, OpenflowExperimenterMessageEvent.createMatcher(connection = connection), ('experimenter', connection), self.messagequeuesize))
        # Add priority to echo reply, or the whole connection is down
        connection.createdqueues.append(connection.scheduler.queue.addSubQueue(\
                self.writepriority + 10, ConnectionWriteEvent.createMatcher(connection = connection, _ismatch = lambda x: hasattr(x, 'echoreply') and x.echoreply), ('echoreply', connection)))
        for m in self.reconnect_init(connection):
            yield m
    def reconnect_init(self, connection):
        connection.xid = ord(os.urandom(1)) + 1
        connection.openflowversion = 0
        connection.openflow_datapathid = None
        connection.openflow_msgbuffer = []
        connection.subroutine(self.handler(connection), asyncStart = False, name = 'handler')
        if False:
            yield
    def closed(self, connection):
        for m in Protocol.closed(self, connection):
            yield m
        connection.handler.close()
        self._logger.info('Openflow connection is closed on %r', connection)
        if connection.openflow_datapathid is not None:
            for m in connection.waitForSend(OpenflowConnectionStateEvent(connection.openflow_datapathid, connection.openflow_auxiliaryid, OpenflowConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
                yield m
    def error(self, connection):
        for m in Protocol.error(self, connection):
            yield m
        connection.handler.close()
        self._logger.warning('Openflow connection is reset on %r', connection)
        if connection.openflow_datapathid is not None:
            for m in connection.waitForSend(OpenflowConnectionStateEvent(connection.openflow_datapathid, connection.openflow_auxiliaryid, OpenflowConnectionStateEvent.CONNECTION_DOWN, connection, connection.connmark, self)):
                yield m
    def _createevent(self, connection, msg):
        if msg.header.type == common.OFPT_ECHO_REQUEST:
            # Direct reply without enqueue
            msg.header.type = common.OFPT_ECHO_REPLY
            echoreply = self.formatreply(msg, msg, connection)
            echoreply.echoreply = True
            return echoreply
        elif connection.openflow_datapathid is None:
            # Connection is pre-setup
            if msg.header.type == common.OFPT_HELLO or msg.header.type == common.OFPT_ERROR:
                return OpenflowPresetupMessageEvent(connection, msg.header.type, connection.connmark, self, message = msg)
            else:
                # Other messages must be parsed in specified version
                ofdef = definations.get(msg.header.version)
                if ofdef is None:
                    # Version is not supported
                    self._logger.warning('Illegal message received from connection %r, message = %r', connection, common.dump(msg))
                    err = common.ofp_error_msg.new()
                    err.header.version = msg.header.version
                    err.type = common.OFPET_BAD_REQUEST
                    err.code = common.OFPBRC_BAD_VERSION
                    err.data = (msg._tobytes())[0:64]
                    write = self.formatreply(err, msg, connection)
                    return write
                elif msg.header.type == ofdef.OFPT_FEATURES_REPLY:
                    return OpenflowPresetupMessageEvent(connection, msg.header.type, connection.connmark, self, message = msg)
                else:
                    # Store other messages
                    connection.openflow_msgbuffer.append(msg)
                    return None
        else:
            if msg.header.version != connection.openflowversion:
                self._logger.warning('Illegal message (version not match) received from connection %r, message = %r', connection, common.dump(msg))
                err = common.ofp_error_msg.new()
                err.header.version = connection.openflowversion
                err.type = common.OFPET_BAD_REQUEST
                err.code = common.OFPBRC_BAD_VERSION
                err.data = (msg._tobytes())[0:64]
                write = self.formatreply(err, msg, connection)
                return write                
            elif msg.header.type == openflow10.OFPT_VENDOR:
                if connection.openflowversion > common.OFP10_VERSION:
                    experimenter = msg.experimenter
                    exptype = msg.exp_type
                else:
                    experimenter = msg.vendor
                    exptype = getattr(msg, 'subtype', 0)
                return OpenflowExperimenterMessageEvent(experimenter, exptype, connection.openflow_datapathid, connection.openflow_auxiliaryid, 
                                                        connection, connection.connmark, self, message = msg)
            elif msg.header.type == common.OFPT_ERROR or msg.header.type in connection.openflowdef.ofp_type_reply_set:
                iserror = (msg.header.type == common.OFPT_ERROR)
                return OpenflowResponseEvent(connection.openflow_datapathid, connection.openflow_auxiliaryid, connection, connection.connmark, msg.header.xid, iserror, self, message = msg)
            elif msg.header.type in connection.openflowdef.ofp_type_asyncmessage_set:
                return OpenflowAsyncMessageEvent(msg.header.type, connection.openflow_datapathid, connection.openflow_auxiliaryid, 
                                                 getattr(msg, 'table_id', 0), getattr(msg, 'cookie', 0), connection, connection.connmark, self, message = msg)
            else:
                # These messages are requests, send a BADREQUEST error
                self._logger.warning('Illegal message (type error) received from connection %r, message = %r', connection, common.dump(msg))
                err = common.ofp_error_msg.new()
                err.header.version = connection.openflowversion
                err.type = common.OFPET_BAD_REQUEST
                err.code = common.OFPBRC_BAD_TYPE
                err.data = (msg._tobytes())[0:64]
                return self.formatreply(err, msg, connection)
    def parse(self, connection, data, laststart):
        start = 0
        events = []
        while True:
            result = common.ofp_msg.parse(data[start:])
            if result is None:
                break
            msg, size = result
            if self.debugging:
                self._logger.debug('message received: %r', common.dump(msg))
            start += size
            e = self._createevent(connection, msg)
            if e is not None:
                events.append(e)
        if laststart == len(data):
            # Remote write close
            events.append(ConnectionWriteEvent(connection, connection.connmark, data = b'', EOF = True))
        return (events, len(data) - start)
    def keepalive(self, connection):
        echo = common.ofp_echo()
        echo.header.version = connection.openflowversion
        try:
            for m in connection.executeWithTimeout(self.keepalivetimeout, self.querywithreply(echo, connection, connection)):
                yield m
            if connection.timeout:
                for m in connection.reset(True):
                    yield m
        except Exception:
            for m in connection.reset(True):
                yield m
    